; totally pipelined assembly code for N element floating point dot product
; using LDDW
; A4 and B4 contain pointers to arrays of floats
; A6 contains an integer with N 
			.def 	_dotproduct4

_dotproduct4:
; cycle 1
			MV		.S1		A6, A1			; copy count to register A1
||			ZERO	.L1		A8				; zero even accumulator
||			ZERO	.L2		B8				; zero odd accumulator
||			LDDW	.D1		*A4++, A7:A6	; get element from first array
||			LDDW	.D2		*B4++, B7:B6	; get element from second array

; cycle 2
			LDDW	.D1		*A4++, A7:A6	; get element from first array
||			LDDW	.D2		*B4++, B7:B6	; get element from second array

; cycle 3
			LDDW	.D1		*A4++, A7:A6	; get element from first array
||			LDDW	.D2		*B4++, B7:B6	; get element from second array

; cycle 4
			LDDW	.D1		*A4++, A7:A6	; get element from first array
||			LDDW	.D2		*B4++, B7:B6	; get element from second array
||	[A1]	SUB		.S1		A1, 2, A1		; decrement counter (NOTE DECREMENT BY 2)

; cycle 5
			LDDW	.D1		*A4++, A7:A6	; get element from first array
||			LDDW	.D2		*B4++, B7:B6	; get element from second array
||	[A1]	B		.S2		LOOP		; conditional branch
||	[A1]	SUB		.S1		A1, 2, A1		; decrement counter (NOTE DECREMENT BY 2)

; cycle 6
			LDDW	.D1		*A4++, A7:A6	; get element from first array
||			LDDW	.D2		*B4++, B7:B6	; get element from second array
||			MPYSP	.M1x	A6, B6, A5		; multiply 
||			MPYSP	.M2x	A7, B7, B5		; multiply 
||	[A1]	B		.S2		LOOP			; conditional branch
||	[A1]	SUB		.S1		A1, 2, A1		; decrement counter (NOTE DECREMENT BY 2)

; cycle 7
			LDDW	.D1		*A4++, A7:A6	; get element from first array
||			LDDW	.D2		*B4++, B7:B6	; get element from second array
||			MPYSP	.M1x	A6, B6, A5		; multiply 
||			MPYSP	.M2x	A7, B7, B5		; multiply 
||	[A1]	B		.S2		LOOP			; conditional branch
||	[A1]	SUB		.S1		A1, 2, A1		; decrement counter (NOTE DECREMENT BY 2)

; cycle 8
			LDDW	.D1		*A4++, A7:A6	; get element from first array
||			LDDW	.D2		*B4++, B7:B6	; get element from second array
||			MPYSP	.M1x	A6, B6, A5		; multiply 
||			MPYSP	.M2x	A7, B7, B5		; multiply 
||	[A1]	B		.S2		LOOP			; conditional branch
||	[A1]	SUB		.S1		A1, 2, A1		; decrement counter (NOTE DECREMENT BY 2)

; cycle 9
			LDDW	.D1		*A4++, A7:A6	; get element from first array
||			LDDW	.D2		*B4++, B7:B6	; get element from second array
||			MPYSP	.M1x	A6, B6, A5		; multiply 
||			MPYSP	.M2x	A7, B7, B5		; multiply 
||	[A1]	B		.S2		LOOP			; conditional branch
||	[A1]	SUB		.S1		A1, 2, A1		; decrement counter (NOTE DECREMENT BY 2)

LOOP:
; cycle 10+
			LDDW	.D1		*A4++, A7:A6	; get element from first array
||			LDDW	.D2		*B4++, B7:B6	; get element from second array
||			MPYSP	.M1x	A6, B6, A5		; multiply 
||			MPYSP	.M2x	A7, B7, B5		; multiply 
||			ADDSP	.L1		A5, A8, A8		; accumulate
||			ADDSP	.L2		B5, B8, B8		; accumulate
||	[A1]	B		.S2		LOOP			; conditional branch
||	[A1]	SUB		.S1		A1, 2, A1		; decrement counter (NOTE DECREMENT BY 2)
; branch occurs here

; epilogue

			ADDSP	.L1x	A8, B8, A0

			ADDSP	.L2x	A8, B8, B0

			ADDSP	.L1x	A8, B8, A0

			ADDSP	.L2x	A8, B8, B0

			NOP

			ADDSP	.L1x	A0, B0, A5

			NOP

			ADDSP	.L2x	A0, B0, B5

			NOP		

			B		B3					; branch back to calling function

			NOP

			ADDSP	.L1x	A5, B5, A4			; result returned in A4

			NOP		3

			.end